/*----------------------------------------------------------------------------
 * Name:    SystemInitHook.c
 * Purpose: configure system core clock
 *          resolve LPCOpen symbols
 *----------------------------------------------------------------------------*/

#include "chip.h"                       // Device header
#include "RTE_Components.h"             // Component selection

#if defined(RTE_DEVICE_LPCOPEN)
const uint32_t ExtClockIn = 0;
#endif

/* Sets the best FLASH clock arte for the passed frequency */
static void setupFlashClocks(uint32_t freq)
{
  if (freq <= 12000000) {
    Chip_SYSCON_SetFLASHAccess(SYSCON_FLASH_1CYCLE);
  }
  else if (freq <= 30000000) {
    Chip_SYSCON_SetFLASHAccess(SYSCON_FLASH_2CYCLE);
  }
  else if (freq <= 60000000) {
    Chip_SYSCON_SetFLASHAccess(SYSCON_FLASH_3CYCLE);
  }
  else if (freq <= 85000000) {
    Chip_SYSCON_SetFLASHAccess(SYSCON_FLASH_4CYCLE);
  } else {
    Chip_SYSCON_SetFLASHAccess(SYSCON_FLASH_5CYCLE);
  }
}

/* Clock and PLL initialization based on the internal oscillator */
void Chip_SetupIrcClocking(uint32_t iFreq)
{
  PLL_CONFIG_T pllConfig;
  PLL_SETUP_T pllSetup;
  PLL_ERROR_T pllError;

  /* Power up the FRO and set this as the base clock */
  Chip_SYSCON_PowerUp(SYSCON_PDRUNCFG_PD_FRO);

  /* Till the PLL is Up use 12MHz FRO as the base clock */
  Chip_Clock_SetMainClockSource(SYSCON_MAINCLKSRC_FRO12MHZ);

  /* Turn on Async Syscon for Timers */
  Chip_SYSCON_Enable_ASYNC_Syscon(true);
  Chip_Clock_SetAsyncSysconClockSource(SYSCON_ASYNC_FRO12MHZ);

  /* Setup FLASH access */
  setupFlashClocks(iFreq);

  /* Select the PLL input to the IRC */
  Chip_Clock_SetSystemPLLSource(SYSCON_PLLCLKSRC_FRO12MHZ);

  /* Power down PLL to change the PLL divider ratio */
  Chip_SYSCON_PowerDown(SYSCON_PDRUNCFG_PD_SYS_PLL);

  /* Setup PLL configuration */
  pllConfig.desiredRate = iFreq;
  pllConfig.InputRate = 0;
  pllConfig.flags = PLL_CONFIGFLAG_FORCENOFRACT;
  pllError = Chip_Clock_SetupPLLData(&pllConfig, &pllSetup);
  if (pllError == PLL_ERROR_SUCCESS) {
    pllSetup.flags = PLL_SETUPFLAG_WAITLOCK | PLL_SETUPFLAG_ADGVOLT;
    pllError = Chip_Clock_SetupSystemPLLPrec(&pllSetup);
  }

  /* Set system clock divider to 1 */
  Chip_Clock_SetSysClockDiv(1);

  /* Set main clock source to the system PLL. This will drive PLL 
     for the main clock and the system clock */
  Chip_Clock_SetMainClockSource(SYSCON_MAINCLKSRC_PLLOUT);
}

/* ----------------------------------------------------------------------------
   -- SystemInitHook()
   ---------------------------------------------------------------------------- */
void SystemInitHook (void) {

  Chip_SetupIrcClocking (100000000U);      /* Initial internal clocking @100MHz */
}
